# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Utility functions."""

from invenio_communities.communities.resolver import CommunityResolver
from invenio_rdm_records.requests import CommunitySubmission


def get_community_uuid(record):
    """Retrieves community UUID.

    WARNING: This should be replaced by the resolvers.
    """
    community_uuid = None
    parent = record['parent']
    if parent.get('communities'):
        community_uuid = parent['communities']['default']
    elif parent.get('review'):
        review = parent['review']
        if review['type'] == CommunitySubmission.type_id:
            community_uuid = review['receiver'][CommunityResolver.type_id]
    return community_uuid



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test that all export formats are working."""

import pytest
from invenio_cache import current_cache


@pytest.fixture(scope="function")
def cache():
    """Clean cache"""
    try:
        current_cache.clear()
        yield current_cache
    finally:
        current_cache.clear()


def test_export_formats(client, running_app, cache, record):
    """Test that all expected export formats are working."""
    # Expected export formats:
    formats = [
        "json",
        "csl",
        "datacite-json",
        "datacite-xml",
        "dublincore",
    ]
    for f in formats:
        res = client.get(f"/records/{record.id}/export/{f}")
        assert res.status_code == 200



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2021 CERN.
# Copyright (C) 2019-2021 Northwestern University.
# Copyright (C)      2021 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Views related to records and deposits."""

from flask import Blueprint
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError, \
    PIDUnregistered
from invenio_records_resources.services.errors import PermissionDeniedError

from ..searchapp import search_app_context
from .deposits import dashboard, deposit_create, deposit_edit
from .filters import can_list_files, get_scheme_label, has_previewable_files, \
    make_files_preview_compatible, order_entries, pid_url, \
    select_preview_file, to_previewer_files
from .records import not_found_error, record_detail, record_export, \
    record_file_download, record_file_preview, record_from_pid, \
    record_latest, record_permission_denied_error, record_tombstone_error


#
# Registration
#
def create_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config.get("APP_RDM_ROUTES")

    blueprint = Blueprint(
        "invenio_app_rdm_records",
        __name__,
        template_folder="../templates",
    )

    # Record URL rules
    blueprint.add_url_rule(
        routes["record_detail"],
        view_func=record_detail,
    )

    blueprint.add_url_rule(
        routes["record_latest"],
        view_func=record_latest,
    )

    app_ext = app.extensions['invenio-rdm-records']
    with app.app_context():
        schemes = app_ext.records_service.config.pids_providers.keys()
    schemes = ','.join(schemes)

    if schemes:
        blueprint.add_url_rule(
            routes["record_from_pid"].format(schemes=schemes),
            view_func=record_from_pid,
        )

    blueprint.add_url_rule(
        routes["record_export"],
        view_func=record_export,
    )

    blueprint.add_url_rule(
        routes["record_file_preview"],
        view_func=record_file_preview,
    )

    blueprint.add_url_rule(
        routes["record_file_download"],
        view_func=record_file_download,
    )

    # Deposit URL rules
    blueprint.add_url_rule(
        routes["dashboard_home"],
        view_func=dashboard,
    )

    blueprint.add_url_rule(
        routes["dashboard_item"],
        view_func=dashboard,
    )

    blueprint.add_url_rule(
        routes["deposit_create"],
        view_func=deposit_create,
    )

    blueprint.add_url_rule(
        routes["deposit_edit"],
        view_func=deposit_edit,
    )

    # Register error handlers
    blueprint.register_error_handler(PIDDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)
    blueprint.register_error_handler(PIDUnregistered, not_found_error)
    blueprint.register_error_handler(KeyError, not_found_error)
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error)

    # Register template filters
    blueprint.add_app_template_filter(can_list_files)
    blueprint.add_app_template_filter(make_files_preview_compatible)
    blueprint.add_app_template_filter(pid_url)
    blueprint.add_app_template_filter(select_preview_file)
    blueprint.add_app_template_filter(to_previewer_files)
    blueprint.add_app_template_filter(has_previewable_files)
    blueprint.add_app_template_filter(order_entries)
    blueprint.add_app_template_filter(get_scheme_label)

    # Register context processor
    blueprint.app_context_processor(search_app_context)

    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Request views module."""

from flask import current_app, g, render_template
from flask_login import login_required
from invenio_rdm_records.proxies import current_rdm_records_service
from invenio_rdm_records.resources.serializers import UIJSONSerializer
from invenio_requests.views.decorators import pass_request
from jinja2 import TemplateNotFound
from sqlalchemy.orm.exc import NoResultFound


@login_required
@pass_request
def requests_detail(request=None, pid_value=None):
    """Requests detail page."""
    request_dict = request.to_dict()

    default_query_config = dict(
        size=current_app.config['REQUESTS_TIMELINE_PAGE_SIZE']
    )

    # temporarily, until serializers and avatars implemented
    try:
        user_email = request_dict["created_by"]["user"]
        request_dict["created_by"].update({
            "avatar": "/static/images/placeholder.png",
            "full_name": user_email
        })
    except KeyError:
        request_dict["created_by"].update({
            "avatar": "/static/images/placeholder.png",
            "full_name": request_dict["created_by"]["community"]
        })

    topic_record = None
    is_draft = None
    permissions = []

    is_topic_record_type = 'record' in request_dict["topic"]

    if is_topic_record_type:
        try:
            topic_record = current_rdm_records_service.read_draft(
                id_=request_dict["topic"]["record"], identity=g.identity)

        except NoResultFound:
            topic_record = current_rdm_records_service.read(
                id_=request_dict["topic"]["record"], identity=g.identity)

        permissions = topic_record.has_permissions_to(
            ['edit', 'new_version', 'manage',
             'update_draft', 'read_files'])
        topic_record = UIJSONSerializer().serialize_object_to_dict(
            topic_record.data)
        is_draft = topic_record["is_draft"]
        request_dict["topic"] = topic_record

    # end temporary block

    try:
        return render_template(
            f"invenio_requests/{request_dict['type']}/index.html",
            # TODO: implement and use request UI serializer
            request=request_dict,
            record=topic_record,
            is_preview=True,
            is_draft=is_draft,
            permissions=permissions,
            files=[],
            default_query_config=default_query_config
        )

    except TemplateNotFound:
        return render_template(
            "invenio_requests/details/index.html",
            # TODO: implement and use request UI serializer
            request=request_dict,
            record=topic_record,
            default_query_config=default_query_config
        )
